"""
Helpers to work and process the rest requests and responses
"""
from typing import Optional

import flask
from werkzeug.datastructures import ImmutableMultiDict

import portal.service.general
from portal.database import Course, Project, User
from portal.rest.schemas import SCHEMAS
from portal.service.errors import DataMissingError


def parse_request_data(schema=None, action: str = None, resource: str = None,
                       partial=False) -> dict:
    """Parses the request data using schema
    Args:
        schema(Schema): Resource schema
        action(str): Action that is used
        resource(str): Name of the resource
        partial: Whether to ignore missing fields.
        If its value is an iterable, only missing fields listed in that iterable will be ignored.

    Returns(dict): Parsed dictionary
    """
    if schema is None:
        schema = getattr(SCHEMAS, resource)
    if callable(schema):
        schema = schema()
    json_data = require_data(action, resource=resource)
    return schema.load(json_data, partial=partial)[0]


def require_data(action: str, resource: str) -> dict:
    """Parses data from json and if they are missing, throws an exception
    Args:
        action(str): Name of the action
        resource(str): Name of the resource

    Returns(dict): Parsed json data

    """
    json_data = flask.request.get_json()
    if not json_data:
        raise DataMissingError(action=action, resource=resource)
    return json_data


class FlaskRequestArgsHelper:
    def __init__(self, parser: 'FlaskRequestHelper'):
        self._parser = parser

    @property
    def _args(self):
        return self._parser.request_args

    @property
    def _general(self) -> portal.service.general:
        return portal.service.general

    def course(self) -> Optional[Course]:
        course = self._args.get('course')
        if course is None:
            return None
        return self._general.find_course(course)

    def user(self) -> Optional[User]:
        user = self._args.get('user')
        if user is None:
            return None
        return self._general.find_user(user)

    def project(self) -> Optional[Project]:
        project = self._args.get('project')
        if project is None:
            return None
        course = self.course()
        if course is None:
            return None
        return self._general.find_project(course, project)


class FlaskRequestHelper:
    @property
    def args(self) -> FlaskRequestArgsHelper:
        return FlaskRequestArgsHelper(parser=self)

    @property
    def request(self) -> flask.Request:
        return flask.request

    @property
    def request_args(self) -> ImmutableMultiDict:
        return self.request.args
